#include <stdio.h>
#include "matrix.h"
#include "commentator.h"

// -------------------------------------------------
// GMP compatibility 
// -------------------------------------------------
#include "gmp.h"






void AssignInteger(Integer& Res, const mpz_t cc) {
// mpz_set( (mpz_ptr)&(Res.gmp_rep), cc);


    struct Spy {
        struct InHeritsInteger : Integer {
        protected:
            friend class Spy;
        };
    
        void assignprotected( Integer& r, const mpz_t cc ) {
            mpz_set( static_cast<mpz_ptr>(& (static_cast<InHeritsInteger&>(r).gmp_rep)),
                     cc );
        }
    };
    
    Spy().assignprotected( Res, cc );
}
void AssignMPZ(mpz_t& cc, const Integer m) {

    std::string s; Integer2string(s, m);
    mpz_init_set_str( cc, s.c_str(), 10 );
}


// -------------------------------------------------
// Sparse BB
// -------------------------------------------------
#include "LinBox/lin_spv_bb.h" 
#include "LinBox/lin_wrap_c++.h"


typedef SparseBlackBoxDom< OperatorWrapper<long> > SPBBlong;
typedef SPBBlong::Row_t homSV_t;
typedef homSV_t::element  homPair;

extern "C" void MatrixSmithFormValence (Commentator * Comm,
                                        const TSparseMatrixArray * Matrix,
                                        char * File_Name,
                                        TCount MaxRank, TCount Dim,
                                        int Method, int Mode, const mpz_t * coeff,
                                        TSmithForm * SmithForm,
					char * numerator, char * denominator) {


    Invariants<unsigned long long > Inv;
    Integer R; AssignInteger(R, *coeff);
    OperatorWrapper<long> LongDom;
    SPBBlong TheMat( LongDom );
    unsigned long ni, nj;
    
    if (Matrix) {
            // Changing matrix structure
        ni = Matrix->NumRows; nj = Matrix->NumColumns;
        SPBBlong::element TheContainer;
        
        TheContainer.resize(ni);
        for (long i = ni; i--;) { 
            TheContainer[i].resize( Matrix->Rows[i].Count );
            TheContainer[i].reactualsize(nj);
        for (long j = TheContainer[i].size(); j--;)
                TheContainer[i][j] = homPair( Matrix->Rows[i].Components[j].Index, Matrix->Rows[i].Components[j].Val );
        }
        // char * temp_file = tempnam("/tmp","bbx_");
	char * temp_file = tempnam("/tmp","bbx_"); // "/tmp/bbx_XXXXXX"; mkstemp(temp_file);
        TheMat.write(temp_file, TheContainer);
        MatrixSmithFormValenceCPP(*Comm, ni, nj, temp_file, Method, Mode, R, Inv, numerator, denominator);
        
    } else {
        TheMat.read(ni,nj, File_Name);
        MatrixSmithFormValenceCPP(*Comm, ni, nj, File_Name, Method, Mode, R, Inv, numerator, denominator);
    }

    
    SmithForm->Rows = Inv.n_row();
    SmithForm->Columns = Inv.n_col();
    SmithForm->NumberOfInvariantFactors = Inv.size();
    TInvariantFactor * factors = new TInvariantFactor[Inv.size()];
    for (long i=0; i<Inv.size(); ++i) {
        AssignMPZ(factors[i].Coefficient, Integer( Inv.getvalue(i) ));
        factors[i].Count = Inv.getexponent(i);
    }

    SmithForm->InvariantFactors = factors;
};


